;;; -*- Lisp -*-
;;;
;;; tie Common Music and CMN together

(in-package :stella)

(defsyntax cmn :pathname (pathname "aaa.eps") :nicknames '(cmn))

(defclass cmn-stream (event-file)
  ((syntax :initform (find-syntax ':cmn))
   (permissions :initform '(:if-exists :supersede :if-does-not-exist :create
                                       :element-type string-char))))

(defclass cmn-event (note)
  ((name :initarg name :initarg :name)
   (duration :initarg duration :initarg :duration)
   (frequency :initarg frequency :initarg :frequency)))

(defmethod print-object ((object cmn-event) stream)
  (format stream
          "#<CMN~@[ Name:~S~]~@[ Time:~S~]~@[ Dur:~S~]~@[ Freq:~S~]>"
          (slot-value-or-default object 'name)
          (slot-value-or-default object 'time)
          (slot-value-or-default object 'duration)
          (slot-value-or-default object 'frequency)))

(defmethod syntax-default-pathname ((syntax cmn) (stream cmn-stream)) "aaa.eps")

(defmethod open-scorefile-stream ((syntax cmn) &rest args)
  (apply #'open-event-stream 'cmn-stream args))

(defmethod initialize-stream-for-processing ((stream cmn-stream))
  (cmn::init-clm-input)
  (setf (cmn::output-file cmn::*cmn-score*) (namestring (slot-value stream 'pathname)))
)

(defmethod deinitialize-stream-for-processing ((stream cmn-stream))
  (let ((command (concatenate 'string "open " (cmn::output-file cmn::*cmn-score*))))
    (cmn::finish-clm-input cmn::*cmn-score*)
    #+(and NeXT Excl) (excl:shell command))
)

(defun check-for-pitch-name (n)
  (if (numberp n)
      n
    (eval (find-symbol (symbol-name n) :cmn))))
    ;; gad what a kludge! -- user passed 'C4 as stella/CM object and we want the cmn version of C4

(defmethod write-event ((object cmn-event) (stream cmn-stream))
  (format t "write-event calling add-note with ~S~&" object)
  (cmn::add-note cmn::*cmn-score* (slot-value object 'name) 
                 (slot-value object 'time) (slot-value object 'duration)
                 (check-for-pitch-name (slot-value object 'frequency))))

;;;
;;; cobble up a test thread of cmn-events. 
;;;

(eval-when (load eval)

#|
(thread test ()
  (element cmn-event name "foo" rhythm 0 frequency '440 duration .5)
  (element cmn-event name "foo" rhythm 0 frequency '220 duration 5)
  (element cmn-event name "foo" rhythm .5 frequency '261 duration .5)
  (element cmn-event name "foo" rhythm 2 frequency '300 duration 2))
|#
(thread test ()
  (element cmn-event name "foo" rhythm 0 frequency 'C4 duration .5)
  (element cmn-event name "foo" rhythm 0 frequency 'A3 duration 5)
  (element cmn-event name "foo" rhythm .5 frequency 'G4 duration .5)
  (element cmn-event name "foo" rhythm 2 frequency 'DF5 duration 2))

)

